package javax.swing;

import javax.swing.event.RowSorterEvent;
import javax.swing.event.RowSorterListener;
import java.util.List;
import javax.swing.event.EventListenerList;

public abstract class RowSorter<M>
{
    private EventListenerList listenerList;
    
    public RowSorter() {
        this.listenerList = new EventListenerList();
    }
    
    public abstract M getModel();
    
    public abstract void toggleSortOrder(int p0);
    
    public abstract int convertRowIndexToModel(int p0);
    
    public abstract int convertRowIndexToView(int p0);
    
    public abstract void setSortKeys(List<? extends RowSorter.SortKey> p0);
    
    public abstract List<? extends RowSorter.SortKey> getSortKeys();
    
    public abstract int getViewRowCount();
    
    public abstract int getModelRowCount();
    
    public abstract void modelStructureChanged();
    
    public abstract void allRowsChanged();
    
    public abstract void rowsInserted(int p0, int p1);
    
    public abstract void rowsDeleted(int p0, int p1);
    
    public abstract void rowsUpdated(int p0, int p1);
    
    public abstract void rowsUpdated(int p0, int p1, int p2);
    
    public void addRowSorterListener(RowSorterListener l) {
        this.listenerList.add(RowSorterListener.class, l);
    }
    
    public void removeRowSorterListener(RowSorterListener l) {
        this.listenerList.remove(RowSorterListener.class, l);
    }
    
    protected void fireSortOrderChanged() {
        this.fireRowSorterChanged(new RowSorterEvent(this));
    }
    
    protected void fireRowSorterChanged(int[] previousRowIndexToModel) {
        this.fireRowSorterChanged(new RowSorterEvent(this, RowSorterEvent.Type.SORTED, previousRowIndexToModel));
    }
    
    void fireRowSorterChanged(RowSorterEvent rowSorterEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == RowSorterListener.class) {
                ((RowSorterListener)listenerList[i + 1]).sorterChanged(rowSorterEvent);
            }
        }
    }
    
    public static class SortKey
    {
        private int column;
        private SortOrder sortOrder;
        
        public SortKey(int column, SortOrder sortOrder) {
            if (sortOrder == null) {
                throw new IllegalArgumentException("sort order must be non-null");
            }
            this.column = column;
            this.sortOrder = sortOrder;
            return;
        }
        
        public final int getColumn() {
            return this.column;
        }
        
        public final SortOrder getSortOrder() {
            return this.sortOrder;
        }
        
        @Override
        public int hashCode() {
            return 37 * (37 * 17 + this.column) + this.sortOrder.hashCode();
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || (o instanceof RowSorter.SortKey && ((RowSorter.SortKey)o).column == this.column && ((RowSorter.SortKey)o).sortOrder == this.sortOrder);
        }
    }
}
